#include "abxlex.hpp"

%% section 1 code and data tables for DFA go here

///////////////////////////////////////////////////
// public constructor, input from an opened file
//

ABXLex::ABXLex(FILE* inputFile=stdin)
{
    inputType = InputFromFile;
    yyInputFile = inputFile;
    yyInputStream = NULL;
    yyOutputFile = stdout;
    yyBuffer = new char[YY_BUF_SIZE];
    yyStateBuffer = new int[YY_BUF_SIZE+1];
    yySawEndOfFile =1;
    yy_init = 1;
}

///////////////////////////////////////////////////
//    public constructor, input from stream
//

ABXLex::ABXLex(istream* inputStream )
{
    inputType = InputFromStream;
    yyBuffer = new char[YY_BUF_SIZE];
    yyStateBuffer = new int[YY_BUF_SIZE+1];
    yyInputStream = inputStream;
    yyInputFile = NULL;
    yyOutputFile = stdout;
    yySawEndOfFile = 1;
    yy_init = 1;
}

///////////////////////////////////////////////////
// public constructor, input from a character string
//

ABXLex::ABXLex(char* inputString)
{
    inputType = InputFromString;
    yyInputFile = NULL;
    yyInputStream = NULL;
	yyInputString = inputString;
    yyOutputFile = stdout;
    yyBuffer = new char[YY_BUF_SIZE];
    yyStateBuffer = new int[YY_BUF_SIZE+1];
    yySawEndOfFile =1;
    yy_init = 1;
}


///////////////////////////////////////////////////
//    public destructor
//

ABXLex::~ABXLex()
{
    delete[] yyBuffer;
    delete[] yyStateBuffer;
}

///////////////////////////////////////////////////
//     function yyWrap :
//         overwritable by user
//

int ABXLex::yyWrap(void)
{
    return (1);
}

//////////////////////////////////////////////////
//    function input :
//        fill the input buffer of lexer
//

int ABXLex::input(void)
{
    int c;

    YY_DO_BEFORE_SCAN
    if (yyBufferPtrC == yyBufferPtrE)
    {
        int yy_n_chars;

        if (yySawEndOfFile)
        {
            if (yyWrap())
                return (EOF);
            YY_INIT;
        }

        yyBufferPtrB = 0;
        YY_INPUT(yyBuffer, yy_n_chars, YY_MAX_LINE);

        if (yy_n_chars == YY_NULL)
        {
            yySawEndOfFile = 1;

            if (yyWrap())
                return (EOF);

            YY_INIT;
            return (input());
        }

        yyBufferPtrC = -1;
        yyBufferPtrE = yy_n_chars - 1;
    }

    c = yyBuffer[++yyBufferPtrC];

    YY_DO_BEFORE_ACTION;

    return (c);
}


void ABXLex::unput(int c)
{
    YY_DO_BEFORE_SCAN;

    if (yyBufferPtrC == 0)
    {
        register int i;
        register int yy_buf_pos = YY_BUF_MAX;
        for (i = yyBufferPtrE; i >= yyBufferPtrC; --i)
        {
            yyBuffer[yy_buf_pos] = yyBuffer[i];
            yyStateBuffer[yy_buf_pos] = yyStateBuffer[i];
            --yy_buf_pos;
        }

        yyBufferPtrC = YY_BUF_MAX - yyBufferPtrE;
        yyBufferPtrE = YY_BUF_MAX;
    }

    if (yyBufferPtrC <= 0)
        YY_FATAL_ERROR("L0027: PCLEX scanner push-back overflow");

    if (yyBufferPtrC >= yyBufferPtrB && yyBuffer[yyBufferPtrC] == '\n')
        yyBuffer[yyBufferPtrC - 1] = '\n';

    yyBuffer[yyBufferPtrC--] = c;

    YY_DO_BEFORE_ACTION;
}

/////////////////////////////////////////////////
//    function yyLex :
//        lexical analyzer
//

int ABXLex::yyLex(void)
{
    register int yy_curst;
    register unsigned char yy_sym;
    int yy_n_chars,
        yy_lp,
        yy_iii,
        yy_buf_pos,
        yy_act,
        yy_full_match;

%% user's declarations go here

    if (yy_init)
    {
        YY_INIT;
        yyStart = 1;
        yy_init = 0;
    }
    goto get_next_token;

do_action:
    yy_full_match = yyBufferPtrC;

    for ( ; ; )
    {
        YY_DO_BEFORE_ACTION

        switch (yy_act)
        {

%% actions go here

            case YY_NEW_FILE:
                break;
            case YY_END_TOK:
                return (YY_END_TOK);
            default:
                YY_FATAL_ERROR("fatal pclex scanner internal error");
        }

get_next_token:
        YY_DO_BEFORE_SCAN

        yy_curst = yyStart;

        if (yyBuffer[yyBufferPtrC] == '\n')
            ++yy_curst;

        yyBufferPtrB = yyBufferPtrC + 1;
        do
        {
            if (yyBufferPtrC == yyBufferPtrE)
            {
                if (yyBufferPtrE >= YY_BUF_LIM)
                {
                    yy_n_chars = yyBufferPtrE - yyBufferPtrB;

                    if (yy_n_chars >= 0)
                    {
                        for (yy_iii = 0; yy_iii <= yy_n_chars; ++yy_iii)
                        {
                            yy_buf_pos = yyBufferPtrB + yy_iii;
                            yyBuffer[yy_iii] = yyBuffer[yy_buf_pos];
                            yyStateBuffer[yy_iii] = yyStateBuffer[yy_buf_pos];
                        }
                    }

                    yyBufferPtrB = 0;
                    yyBufferPtrE = yy_n_chars;

                    if (yyBufferPtrE >= YY_BUF_LIM)
                        YY_FATAL_ERROR("pclex input buffer overflowed");

                    yyBufferPtrC = yyBufferPtrE;
                }   
                else
                    if (yySawEndOfFile)
                    {
saweof:
                        if (yyBufferPtrB <= yyBufferPtrE)
                            break;
                        if (yyWrap())
                            yy_act = YY_END_TOK;
                        else
                        {
                            YY_INIT;
                            yy_act = YY_NEW_FILE;
                        }
                        goto do_action;
                    }

                YY_INPUT((yyBuffer + yyBufferPtrC + 1), yy_n_chars, YY_MAX_LINE);
                if (yy_n_chars == YY_NULL)
                {
                    if (yySawEndOfFile)
                        YY_FATAL_ERROR("pclex scanner saw EOF twice - shouldn't happen");
                    yySawEndOfFile = 1;
                    goto saweof;
                }

                yyBufferPtrE += yy_n_chars;
            }
            ++yyBufferPtrC;
            yy_sym = yyBuffer[yyBufferPtrC];
            if (yy_sym == '\000')
                yy_sym = '\001';
         

            while (c[b[yy_curst] + yy_sym] != yy_curst)
                yy_curst = d[yy_curst];

            yy_curst = n[b[yy_curst] + yy_sym];
            yyStateBuffer[yyBufferPtrC] = yy_curst;
        }
        while (b[yy_curst] != YY_JAM_BASE);

        if (yyBufferPtrC >= yyBufferPtrB)
        {
            yy_curst = yyStateBuffer[yyBufferPtrC];
            yy_lp = l[yy_curst];

find_rule:
            for ( ; ; )
            {
                if (yy_lp && yy_lp < l[yy_curst + 1])
                {
                    yy_act = a[yy_lp];
                    goto do_action;
                }

                if (--yyBufferPtrC < yyBufferPtrB)
                    break;

                yy_curst = yyStateBuffer[yyBufferPtrC];
                yy_lp = l[yy_curst];
            }
        }
        ++yyBufferPtrC;
        yy_act = YY_DO_DEFAULT;
    }
}

