
/*******************************************************************
*                                                                  *
*  lex.l: scanner description for Simple Calculator                *
*                                                                  *
*  By Y. Jenny Luo                                                 *
*                                                                  *
*  PCYACC (R) is a software product of ABRAXAS SOFTWARE INC.       *
*  Copyright (C) 1986-1996 by ABRAXAS SOFTWARE INC.                *
*                                                                  *
*******************************************************************/

%{

/*******************************************************************
*                                                                  *
*  lex.cpp: lexical analyzer for Simple Calculator                 *
*                                                                  *
*  By Y. Jenny Luo                                                 *
*                                                                  *
*  PCYACC (R) is a software product of ABRAXAS SOFTWARE INC.       *
*  Copyright (C) 1986-1996 by ABRAXAS SOFTWARE INC.                *
*                                                                  *
*******************************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "pcl_sk.hpp"
#include "public.h"             /* define global information */
#include "dcalc.h"               /* header file generated by PCYACC */

#define LEX_DEBUG       1
#define YYERRCODE       256


unsigned char yytext[TEXTLEN];   /* current token's text */
char yyerrsrc[100];         
extern int yyerrcnt;            /* count of error */

FILE *yyerrfile = stderr;

char* yydisplay(int tokenvalue); /* display the token based on tokenvalue */
void yyerror(char *s, char *t, char *YYTEXT);
void errprefix(char *msg, char *YYTEXT); /* print where and what error
				     message on yyerrfile */
int search(char *str);      /* transfer all text string into uppercase and
                               return corresponding token value */

int yylineno = 1;              /* current line number */

%}

letter			[a-zA-Z_]
alphanum		[a-zA-Z_0-9]
digit           [0-9]
blank           [ \t]
sign            [+-/*]
other           .

%%

{letter}{alphanum}*	{
					    yylval.str = yytext;
						return search(yytext);
					}
{digit}+			{
						sscanf(yytext, "%lf", &yylval.db);
						yylval.str = NULL;
						return NUMBER;
					}

{digit}+\.{digit}*	{
						sscanf(yytext, "%lf", &yylval.db);
						yylval.str = NULL;
						return NUMBER;
					}

\.{digit}+			{
						sscanf(yytext, "%lf", &yylval.db);
						yylval.str = NULL;
						return NUMBER;
					}

sign				return *yytext;  

\n					{
						yylineno++;
						return '\n';
					}  

{blank}+			;

{other}				return yytext[0];

%%

/*********************************************************************
*  FUNCTION: yydisplay                                               *
*  PARAMETER: int                                                    *
*  RETURN: int                                                       *
*  DESCRIPTION: display the token based on the input tokenvalue      *
*********************************************************************/
char* yydisplay(int tokenvalue)
{
	int i;
	static char buf[TEXT];

	if (tokenvalue==NUMBER)
		return "NUMBER";
    
	if (tokenvalue==IDENTIFIER)
		return "IDENTIFIER";

    if (tokenvalue==SQRT)
	    return "SQRT";

	switch (tokenvalue)
	{
		case '\a':  return ("[\\a]");
		case '\b':  return ("[\\b]");
		case '\v':  return ("[\\v]");
		case '\f':  return ("[\\f]");
		case '\r':  return ("[\\r]");
		case '\t':  return ("[\\t]");
		case '\n':  return ("[\\n]");
	}

	if (tokenvalue == 0)
	{
		return ("[end of file]");
	}
	else if (tokenvalue == YYERRCODE)
	{
		return ("[error]");
	}

	if (isprint(tokenvalue))
	{
		sprintf(buf, "%c", tokenvalue);
		return buf;
	}
	else
	{
		sprintf(buf, "[char %d]", tokenvalue);
		return buf;
	}

}

/************************************************************************
*  FUNCTION:    errprefix                                               *
*  PARAMETER:   char *                                                  *
*  RETURN:      none                                                    *
*  DESCRIPTION: print where and what error message on error file        *
************************************************************************/
void errprefix(char *msg, char *YYTEXT)
{
    int punct = 0;

    fprintf(yyerrfile, "[error %d] ", yyerrcnt+1);

    if (yyerrsrc[0] != '\0')                        /* any input file name? */
    {
		fprintf(yyerrfile, "file '%s'", yyerrsrc);
		punct = 1;
    }

    if (yylineno >= 0)                                /* valid line number? */
    {
		if (punct)
			fprintf(yyerrfile, ", ");

		fprintf(yyerrfile, "line %d", yylineno);
		punct = 1;
    }

    if (YYTEXT != NULL && *YYTEXT != '\0')                   /* real token? */
    {
		if (punct)                                                                                                                                                                                                                                              
			fprintf(yyerrfile, " ");

		fprintf(yyerrfile, "near \"%s\"", YYTEXT);
		punct = 1;
    }

    if (punct)
		fprintf(yyerrfile, ": ");

    fprintf(yyerrfile, "%s\n", msg);
}

/************************************************************************
*  FUNCTION:    yyerror                                                 *
*  PARAMETER:   char *,  char *                                         *
*  RETURN:      none                                                    *
*  DESCRIPTION: for improved error reporting                            *
************************************************************************/
void yyerror(char *s, char *t, char *YYTEXT)
{
    static const char expecting[] = "expecting: ";
    static int list = 0;
    static int column = 0;

    if (s != NULL)
    {
		if (column != 0)
			fputc('\n', yyerrfile);
	
		errprefix(s, YYTEXT);

		if (t == NULL)
			column = 0;
		else
			column = fprintf(yyerrfile, "actual: %s", t);
		list = 0;
    }
    else
    {
	    if (t!=NULL)
		{
			if (list == 0)
			{
				if (column + strlen(t) + sizeof(expecting) + 1 < LINEWIDTH - 2)
					column += fprintf(yyerrfile, "  %s%s", expecting, t);
				else
					column = fprintf(yyerrfile, "\n%s%s", expecting, t) - 1;
			}
			else
			{
				if (column + strlen(t) < LINEWIDTH - 2)
					column += fprintf(yyerrfile, ", %s", t);
				else
					column = fprintf(yyerrfile, ",\n    %s", t) - 1;
			}
			++list;
		}
		else
		{
			fprintf(yyerrfile, "\n");
			column = list = 0;
		}
	}
}

int search(char *str)
{
  int i = 0, ch;

  ch = str[i];
  while (ch!='\0')
  {
    if (ch<='z' && ch>='a')
	  str[i] = toupper(ch);
	i++;
	ch = str[i];
  }
  if (strcmp(str, "SQRT")==0)
    return SQRT;
  else
    return IDENTIFIER;  
}