/**********************************************************
 |  Windows PIC (c) Copyright 1996 by Abraxas Software.   |
 |  Adaptation by Abraxas PCYACC licensees is permitted.  |
 |                                                        |
 |  This program demonstrates the use of the TParser      |
 |  class with Abraxas PCYACC. It is designed to be       |
 |  run under Windows 3.1. Use the Visual C++ compiler    |
 |  and MFC 1.0 (Microsoft Foundation Classes )           |                                      |
 |                                                        |
 |  A command line like this will create grammar.cpp:     |
 |  pcyacc -pParser.h -Dpic.h -Cgrammar.cpp -v grammar.y  |
 |                                                        |
 |  The file tparser.h contains the TParser interface.    |
 |  You may derive your own parser classes from TParser.  |
 **********************************************************/
       
       
#include "stdafx.h"
#include "resource.h"
#include "pic.h"

/////////////////////////////////////////////////////////////////////////////

// theApp:
// Just creating this application object runs the whole application.
//
CTheApp NEAR theApp;

/////////////////////////////////////////////////////////////////////////////

// CMainWindow constructor:
// Create the window with the appropriate style, size, menu, etc.
//
CMainWindow::CMainWindow()
{
    LoadAccelTable( "MainAccelTable" );
	Create( NULL, "MFC Pic",
		    WS_OVERLAPPEDWINDOW, rectDefault, NULL, "MainMenu" );

	State.PenSize           = 3;
	State.PenColor          = RGB_GRAY;
	State.PenDotted         = FALSE;
	State.BrushColor        = RGB_GRAY;
	
    char FilterString[] = "Pic Files (*.pic)\0*.pic\0\0";
    CFileDialog dlg(TRUE);
    dlg.m_ofn.lpstrFilter = (LPSTR) FilterString;
    if (dlg.DoModal() == IDOK) {
    TParser Parser( (const char*) dlg.GetFileTitle() ); 
		if ( Parser.yyparse() )
			{
			MessageBox( " Parse failed. ", "    PIC Message     ", MB_OK );
			exit( 2 );
			}
    }
    
    Drawing = FALSE;

}

CMainWindow::~CMainWindow()
	{ 

	}


// CMainWindow message map:


BEGIN_MESSAGE_MAP( CMainWindow, CFrameWnd )
	//{{AFX_MSG_MAP( CMainWindow )
	ON_WM_SIZE()
	ON_WM_PAINT() 
	ON_COMMAND( IDM_ABOUT, OnAbout )
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPicWin message handlers 

void CMainWindow::OnAbout()
{
	CDialog about( "AboutBox", this );
	about.DoModal();
}

void CMainWindow::OnPaint()
{
	CPaintDC dc(this); // device context for painting
	
	// TODO: Add your message handler code here
	
	// Do not call CWnd::OnPaint() for painting messages 

      for ( int i = 0; i < ocount; i++ )
		{
		State.PenColor = objlst[i]->color;
		State.BrushColor = State.PenColor;
		State.Filled = FALSE;
		State.PenDotted = FALSE;
		if ( objlst[i]->style == SOLID )
			{
			State.Filled = TRUE;
			}
		else if ( objlst[i]->style == DOTTED )
			{
			State.PenDotted = TRUE;
			}

		switch ( objlst[i]->shape )
			{
		case LINE:
			DrawLine( objlst[i]->npoints, objlst[i]->x_coord, objlst[i]->y_coord );
			break;
		case POLYGON:
			DrawPoly( objlst[i]->npoints, objlst[i]->x_coord, objlst[i]->y_coord );
			break;
		case BOX:
			DrawRect( objlst[i]->npoints, objlst[i]->x_coord, objlst[i]->y_coord );
			break;
		case ELLIPSE:
			DrawOval( objlst[i]->npoints, objlst[i]->x_coord, objlst[i]->y_coord );
			break;
		default:
			MessageBox( "Unknown object.", "   PIC   ", MB_OK );
			break;
			}
		}

}

void
CMainWindow::DrawLine( int npts, int x[], int y[] )
	{ 
    
    CClientDC dc(this);
	
	int lineStyle = ( State.PenDotted ? PS_DOT : PS_SOLID );
	int lineSize  = ( State.PenDotted ? 1 : State.PenSize );
	
    CPen *pOldPen;
	CPen pPen( lineStyle, lineSize, State.PenColor );
    pOldPen = dc.SelectObject( &pPen );

	dc.MoveTo( x[0], y[0] );
	for ( int i = 1; i < npts; i++ )
		dc.LineTo( x[i], y[i] );

	dc.SelectObject( pOldPen );
	
	}

void
CMainWindow::DrawRect( int npts, int x[], int y[] )
	{
	
	CClientDC dc(this);
	
	if ( npts != 2 )
		exception( "Wrong number of points for DrawRect" );

	int lineStyle = ( State.PenDotted ? PS_DOT : PS_SOLID );
	int lineSize  = ( State.PenDotted ? 1 : State.PenSize );

    CPen *pOldPen;
	CPen pPen( lineStyle, lineSize, State.PenColor );
    pOldPen = dc.SelectObject( &pPen );

	if ( State.Filled )     // Draw filled figure
		{
		CBrush* pOldBrush = NULL;
		CBrush* pBrush = new CBrush;
		pBrush->CreateSolidBrush( State.BrushColor );
		dc.SelectObject( pBrush );
	
		dc.Rectangle( x[0], y[0], x[1], y[1] );
	
	    dc.SelectObject( pOldBrush );
		delete pBrush;
		}
	else                    // Draw outlined figure
		{

		dc.Rectangle( x[0], y[0], x[1], y[1] );

		} 

	dc.SelectObject( pOldPen );
}

void
CMainWindow::DrawOval( int npts, int x[], int y[] )
	{ 
	CClientDC dc(this);
	
	if ( npts != 2 )
		exception( "Wrong number of points for DrawOval" );

	int lineStyle = ( State.PenDotted ? PS_DOT : PS_SOLID );
	int lineSize  = ( State.PenDotted ? 1 : State.PenSize );
    
    CPen *pOldPen;
	CPen pPen( lineStyle, lineSize, State.PenColor );
    pOldPen = dc.SelectObject( &pPen );


	if ( State.Filled )     // Draw filled ellipse
		{
		CBrush* pOldBrush = NULL;
		CBrush* pBrush = new CBrush;
		pBrush->CreateSolidBrush( State.BrushColor );
		dc.SelectObject( pBrush );
	
		dc.Ellipse( x[0], y[0], x[1], y[1] );
		
		dc.SelectObject( pOldBrush );
		delete pBrush;
		}
	else                    // Draw outlined ellipse
		{
		dc.SelectObject( GetStockObject(NULL_BRUSH) );
		dc.Ellipse( x[0], y[0], x[1], y[1] );
		}

	dc.SelectObject( pOldPen );
}

void
CMainWindow::DrawPoly( int npts, int x[], int y[] )
	{ 
	
	CClientDC dc(this);
	
	if ( npts < 3 )
		exception( "DrawPoly requires at least 3 points." );

	POINT *polypoints = new POINT[npts];
	for ( int i = 0; i < npts; i++ )
		{
		polypoints[i].x = x[i];
		polypoints[i].y = y[i];
		}

	int lineStyle = ( State.PenDotted ? PS_DOT : PS_SOLID );
	int lineSize  = ( State.PenDotted ? 1 : State.PenSize );
	
	CPen *pOldPen;
	CPen pPen( lineStyle, lineSize, State.PenColor );
    pOldPen = dc.SelectObject( &pPen );

	if ( State.Filled )
		{
		CBrush* pOldBrush = NULL;
		CBrush* pBrush = new CBrush;
		pBrush->CreateSolidBrush( State.BrushColor );
		dc.SelectObject( pBrush );
		
		dc.Polygon(  polypoints, npts );
		
		dc.SelectObject( pOldBrush );
		delete pBrush;
		}
	else
		dc.Polyline( polypoints, npts );

	dc.SelectObject( pOldPen );

	delete polypoints;
}

void CMainWindow::OnSize(UINT nType, int cx, int cy)
{
	CWnd::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here  

   //  For now we don't change anything here
	
}

/////////////////////////////////////////////////////////////////////////////
// CTheApp

// InitInstance:

BOOL CTheApp::InitInstance()
{

	SetDialogBkColor();     // hook gray dialogs (was default in MFC V1)

	m_pMainWnd = new CMainWindow();
	m_pMainWnd->ShowWindow( m_nCmdShow );
	m_pMainWnd->UpdateWindow();

	return TRUE;
}
