/**********************************************************************
*                                                                     *
*  ABXSYM.CPP: SYMBOL TABLE CLASS FOR PCYACC OO CLASS LIBRARY         *
*                                                                     *
*  Version 1.0                                                        *
*                                                                     *
*  By Y. Jenny Luo                                                    *
*                                                                     *
*  PCYACC (R) is a software product of ABRAXAS SOFTWARE INC.          *
*  Copyright (C) 1986-1996 by ABRAXAS SOFTWARE INC.                   *
*                                                                     *
*  Reference: PCYACC Object Oriented Toolkit                          *
*             ABRAXAS SOFTWARE, INC                                   *
*                                                                     *
**********************************************************************/
#include "abxsym.hpp"


ABXSymbolTable::ABXSymbolTable()
{
	symtab_head = NULL;
	symtab_ptr = NULL;
}

ABXSymbolTable::~ABXSymbolTable()
{
//	free(symtab_head);
//	free(symtab_ptr);
}

/* Constructs a symbol table with only scope 0 (global layer). */	
void ABXSymbolTable::initSymbolTable(void)
{
	scope_indicator = 0;
	symtab_head = (struct ABXsymtab *)malloc(sizeof(struct ABXsymtab));
	symtab_ptr = symtab_head;
	symtab_ptr->scope = 0;
	symtab_ptr->symbol_list = NULL;
	symtab_ptr->next = NULL;
}

/* Sets the current scope indicator to the scp and current symtab 
pointer has been changed accordingly. If the specified scope
does not exist, the function returns -1. Otherwise, it returns the 
scope number just set. */
int ABXSymbolTable::setScope(int scp)
{
	struct ABXsymtab *ptr;
	if (scp < 0)
		return -1;
	else
	{
		ptr = symtab_head;
		while (ptr->scope < scp)
		{
			ptr = ptr->next;
			if (ptr == NULL)
				return -1;
		}
		if (ptr->scope == scp)
		{
			scope_indicator = ptr->scope;
			symtab_ptr = ptr;
			return scope_indicator;
		}
		if (ptr->scope > scp)
		{
			return -1;
		}			
	}	

	return -1;
}

/* Gets the value of current scope indicator. */
int ABXSymbolTable::getScope(void)
{
	return scope_indicator;
}

/* Gets the biggest scope number of the symbol table. */
int ABXSymbolTable::maxScope(void)
{
	struct ABXsymtab *ptr, *prev;

	ptr = symtab_head;
	while (ptr != NULL)
	{
		prev = ptr;
		ptr = ptr->next;
	}
	return prev->scope;
}

/* Creates a new scope, set current scope indicator and 
symbol table pointer. If successful, function returns 1, 
if the scope requested already exists returns 2, otherwise
returns 0.*/
int ABXSymbolTable::createScope(int scp)
{	
	struct ABXsymtab *ptr, *tmp, *prev;

	if (scp<0)
		return 0;
	ptr = (struct ABXsymtab *)malloc(sizeof(struct ABXsymtab));
	if (ptr==NULL)
		return 0;	/* memory allocation failure */
	else
	{
		/* set current scope indicator */
		scope_indicator = scp;  
		tmp = symtab_head;
		while (tmp->scope <= scp)
		{
			if (tmp->scope==scp)
			{
				symtab_ptr = tmp; /* set surrent symbol table pointer */
				return 2;		/* scope already exists */
			}
			prev = tmp;       
			tmp = tmp->next;
			/* create new scope at the end of list */
			if (tmp->next==NULL)	
			{
				tmp->next = ptr;
				tmp = tmp->next;
				tmp->scope = scp;
				tmp->symbol_list = NULL;
				tmp->next = NULL;
				symtab_ptr = tmp; /* set surrent symbol table pointer */
				return 1;
			}	
		}
		/* insert new scope in between of list */
		prev = ptr;
		ptr->scope = scp;
		ptr->symbol_list = NULL;
		ptr->next = tmp;
		symtab_ptr = tmp; /* set surrent symbol table pointer */
		return 1;
	}
}

/* Removes all symbols in symbol table with scope scp, set current
scope indicator and current symtab pinter accordingly. If the specified 
scope does not exist, function returns 0, otherwise 1. */
int ABXSymbolTable::cleanScope(int scp)
{
	struct ABXsymtab *ptr, *tmp, *prev;

	if (scp < 0)   return 0;
	ptr = symtab_ptr;    /* current symbol table indicator */
	if (ptr == symtab_head)  /* current symtab pointer = symtab head */
	{
		if (ptr->scope == scp)
		{
			free(ptr);
			initSymbolTable();
			return 1;
		}
	}

	if (ptr->scope < scp)   /* forward searching symbol table */
	{
		while (ptr->scope < scp)
		{
			prev = ptr;
			ptr = ptr->next;
			if (ptr == NULL) /* specified scope does not exist */
			{
				scope_indicator = prev->scope;
				symtab_ptr = prev;
				return 0;
			}
		}
		if (ptr->scope == scp)
		{
			ptr->symbol_list = NULL;
			scope_indicator = ptr->scope;
			symtab_ptr = ptr;
			return 1;
		}
		if (ptr->scope > scp)   /* specified scope does not exist */
		{
			scope_indicator = ptr->scope;
			symtab_ptr = ptr;
			return 0;
		}
	}
	else if (ptr->scope == scp) /* remove current scope from symbol table list */
	{
		ptr = symtab_head;
		while (ptr != symtab_ptr)
		{
			ptr = ptr->next;
		}
		ptr->symbol_list = NULL;
		scope_indicator = ptr->scope;
		symtab_ptr = ptr;
		return 1;
	}
	else if (ptr->scope > scp)   /* backward searching symbol table */
	{
		tmp = ptr;    /* save current symtab pointer to tmp */
		ptr = symtab_head;
		while (ptr->scope < scp )
		{
			prev = ptr;
			ptr = ptr->next;
			if (ptr == NULL)  /* specified scope does not exist */
			{
				symtab_ptr = prev;
				scope_indicator = prev->scope;
				return 0;
			}
			if (ptr == tmp)
			{
				return 0;
			}
		}
		if (ptr->scope == scp)
		{
			ptr->symbol_list = NULL;
			scope_indicator = ptr->scope;
			symtab_ptr = ptr;
			return 1;
		}
		if (ptr->scope > scp)   /* specified scope does not exist */
		{
			scope_indicator = ptr->scope;
			symtab_ptr = ptr;
			return 0;
		}
	}

	return 0;
}

/* Adds a symbol into symbol table according to the input parameters. 
If the new symbol is added successfully, this function returns 1, if
the specified symbol already exists in symbol table, returns 2,
otherwise returns 0. The current scope indicator and symtab pointer
have been modified accordingly. */
int ABXSymbolTable::addSymbol(char *name, int attribute, 
							  int type, ABXVALUE value, int scp)
{
	struct ABXsymtab *ptr, *tmp, *prev;
	struct ABXsymtabentry *symtabentry, *symtabentry_ptr;

	/* create symbol table entry and set member data */
	symtabentry = (struct ABXsymtabentry *)malloc(sizeof(struct ABXsymtabentry));
	strcpy(symtabentry->name, name);
	symtabentry->attribute = attribute;
	symtabentry->type = type;
	symtabentry->value = value;
	symtabentry->scope = scp;
	symtabentry->next = NULL;

	if (scp < 0)   
	{
		free(symtabentry);
		return 0;
	}
	ptr = symtab_ptr;  /* current symbol table indicator */
	if (ptr == symtab_head)  /* current symtab pointer = symtab head */
	{
		if (ptr->scope == scp)
		{
			if (insertSymbol(symtabentry) == 0) /* symbol already exists */
			{
				return 2;
			}
			else
			{
				return 1;   /* successfully add new symbol */
			}
		}
	}

	if (ptr->scope < scp)  /* forward searching symbol table */
	{
		while (ptr->scope < scp)
		{
			prev = ptr;
			ptr = ptr->next;
			if (ptr == NULL) /* specified scope does not exist */
			{
				scope_indicator = prev->scope;
				symtab_ptr = prev;
				free(symtabentry);
				return 0;
			}

		}
		if (ptr->scope == scp)
		{
			symtab_ptr = ptr;
			scope_indicator = ptr->scope;
			if (insertSymbol(symtabentry) == 0)  /* symbol already exists */
			{
				return 2;
			}
			else 
			{
				return 1;
			}			
		}
		if (ptr->scope > scp)  /* specified scope does not exist */
		{
			scope_indicator = ptr->scope;
			symtab_ptr = ptr;
			free(symtabentry);
			return 0;
		}
	}

	else if (ptr->scope == scp) /* add symbol in current symbol table list */
	{
		symtab_ptr = ptr;
		scope_indicator = ptr->scope;
		if (insertSymbol(symtabentry) == 0)  /* symbol already exists*/
			return 2;
		else return 1;
	}

	else if (ptr->scope > scp) /* backward searching symbol table */
	{
		tmp = ptr;  /* save current symtab pointer to tmp */
		ptr = symtab_head;
		while (ptr->scope < scp)
		{
			prev = ptr;
			ptr = ptr->next;
			if (ptr == NULL)  /* specified scope does not exist */
			{
				symtab_ptr = prev;
				scope_indicator = prev->scope;
				free(symtabentry);
				return 0;
			}
			if (ptr == tmp)
			{
				free(symtabentry);
				return 0;
			}
		}
		if (ptr->scope == scp)
		{
			scope_indicator = ptr->scope;
			if (insertSymbol(symtabentry) == 0)  /* symbol already exists*/
				return 2;
			else return 1;
		}
		if (ptr->scope > scp) /* specified scope does not exist */
		{
			scope_indicator = ptr->scope;
			symtab_ptr = ptr;
			free(symtabentry);
			return 0;
		}
	}

	return 0;
}

/* Looks up a symbol that is already inserted in the symbol table. 
If the symbol is found, returns the symbol's symbol table entry 
pointer, otherwise returns NULL. The current scope indicator and 
symtab pointer have been modified accordingly.*/
struct ABXsymtabentry * ABXSymbolTable::lookupSymbol(char *s, int scp)
{
	struct ABXsymtab *ptr, *tmp, *prev;
	struct ABXsymtabentry *symtabentry;

	if (scp < 0)   return 0;
	ptr = symtab_ptr;    /* current symbol table indicator */
	if (ptr == symtab_head)  /* current symtab pointer = symtab head */
	{
		if (ptr->scope == scp)
		{
			symtabentry = ptr->symbol_list;
			if (symtabentry==NULL)
				return NULL;
			while (strcmp(symtabentry->name, s) != 0)
			{
				symtabentry = symtabentry->next;
				if (symtabentry == NULL)
					return NULL;
			}
			return symtabentry;
		}
	}

	if (ptr->scope < scp)   /* forward searching symbol table */
	{
		while (ptr->scope < scp)
		{
			prev = ptr;
			ptr = ptr->next;
			if (ptr == NULL) /* specified scope does not exist */
			{
				scope_indicator = prev->scope;
				symtab_ptr = prev;
				return NULL;
			}
		}
		if (ptr->scope == scp)
		{
			symtabentry = ptr->symbol_list;
			while (strcmp(symtabentry->name, s) != 0)
			{
				symtabentry = symtabentry->next;
				if (symtabentry == NULL)
					return NULL;
			}
			symtab_ptr = ptr;
			scope_indicator = ptr->scope;
			return symtabentry;
		}
		if (ptr->scope > scp)   /* specified scope does not exist */
		{
			scope_indicator = ptr->scope;
			symtab_ptr = ptr;
			return NULL;
		}
	}
	else if (ptr->scope == scp) /* corresponding scope has been found */
	{
		symtabentry = ptr->symbol_list;
		while (strcmp(symtabentry->name, s) != 0)
		{
			symtabentry = symtabentry->next;
			if (symtabentry == NULL)
				return NULL;
		}
		symtab_ptr = ptr;
		scope_indicator = ptr->scope;
		return symtabentry;
	}
	else if (ptr->scope > scp)   /* backward searching symbol table */
	{
		tmp = ptr;    /* save current symtab pointer to tmp */
		ptr = symtab_head;
		while (ptr->scope < scp )
		{
			prev = ptr;
			ptr = ptr->next;
			if (ptr == NULL)  /* specified scope does not exist */
			{
				symtab_ptr = prev;
				scope_indicator = prev->scope;
				return NULL;
			}
			if (ptr == tmp)
			{
				return NULL;
			}
		}
		if (ptr->scope == scp)
		{
			symtabentry = ptr->symbol_list;
			while (strcmp(symtabentry->name, s) != 0)
			{
				symtabentry = symtabentry->next;
				if (symtabentry == NULL)
					return NULL;
			}
			symtab_ptr = ptr;
			scope_indicator = ptr->scope;
			return symtabentry;
		}
		if (ptr->scope > scp)   /* specified scope does not exist */
		{
			scope_indicator = ptr->scope;
			symtab_ptr = ptr;
			return NULL;
		}
	}
	return NULL;
}

/* deletes a symbol table entry according to its name and scope.
return 1 if the symbol is deleted successfully. If the symbol
does not exist, return 0. The current scope indicator and symtab 
pointer have been modified accordingly.*/
int ABXSymbolTable::deleteSymbol(char *name, int scp)
{
	struct ABXsymtabentry *symtabentry, *ptr, *prev;

    symtabentry = lookupSymbol(name, scp);
	if (symtabentry == NULL)  return 0;
	ptr = symtab_ptr->symbol_list;  
	while (ptr != symtabentry)
	{
		prev = ptr;
		ptr = ptr->next;
	}
	prev->next = ptr->next;
	free(symtabentry);
	return 1;
}

/* Sets the symbol attribute value (defined by user) according to
its name scope. This function will return 1 if successful, 
otherwise return 0. The current scope indicator and symtab pointer
have been modified accordingly.*/
int ABXSymbolTable::setSymbolAttribute(char *name, int scp, int attribute)
{
	struct ABXsymtabentry *symtabentry;

	symtabentry = lookupSymbol(name, scp);
	if (symtabentry == NULL)
		return 0;
	else
	{
		symtabentry->attribute = attribute;
		return 1;
	}
}

/* Sets the symbol data type according to its name and scope. 
Customers can define their own types by using macro definitions 
according to their specific compiler requirements. This function 
will return 1 if successful, otherwise, return 0. The current scope 
indicator and symtab pointer have been modified accordingly.*/
int ABXSymbolTable::setSymbolType(char *name, int scp, int type)
{
	struct ABXsymtabentry *symtabentry;

	symtabentry = lookupSymbol(name, scp);
	if (symtabentry == NULL)
		return 0;
	else
	{
		symtabentry->type = type;
		return 1;
	}
}

/* Sets the symbol value according to its name and scope. Return
1 if proper entry has been found, otherwise, return -1. The current 
scope indicator and symtab pointer have been modified accordingly.*/
int ABXSymbolTable::setSymbolValue(char *name, int scp, ABXVALUE value)
{
	struct ABXsymtabentry *symtabentry;

	symtabentry = lookupSymbol(name, scp);
	if (symtabentry == NULL)
		return 0;
	else
	{
		symtabentry->value = value;
		return 1;
	}
}

/* Returns a symbol's attribute if the symbol was found, otherwise
returns -1. The current scope indicator and symtab pointer
have been modified accordingly.*/
int ABXSymbolTable::getSymbolAttribute(char *name, int scp)
{
	struct ABXsymtabentry *symtabentry;

	symtabentry = lookupSymbol(name, scp);
	if (symtabentry == NULL)
		return 0;
	else
	{
		return symtabentry->attribute;
	}
}

/* Returns a symbol's type if the symbol was found, otherwise
returns -1. The current scope indicator and symtab pointer
have been modified accordingly.*/
int ABXSymbolTable::getSymbolType(char *name, int scp, int type)
{
	struct ABXsymtabentry *symtabentry;

	symtabentry = lookupSymbol(name, scp);
	if (symtabentry == NULL)
		return 0;
	else
	{
		return symtabentry->type;
	}
}

/* Returns a symbol's value if the symbol was found, otherwise 
returns -1. The current scope indicator and symtab pointer
have been modified accordingly.*/
union ABXVALUE ABXSymbolTable::getSymbolValue(char *name, int scp, 
										ABXVALUE value)
{
	struct ABXsymtabentry *symtabentry;

	symtabentry = lookupSymbol(name, scp);
	return symtabentry->value;
}

/* Insert symbol based on the current scope indicator and symbol
table pointer. If success, return 1. Otherwise, return 0
(symbol already exists). */
int ABXSymbolTable::insertSymbol(ABXsymtabentry *symbol)
{
	struct ABXsymtabentry *ptr, *prev;

	/* no symbol exists in current scope */
	if (symtab_ptr->symbol_list == NULL)
	{
		symtab_ptr->symbol_list = symbol;
		return 1;
	}

	ptr = symtab_ptr->symbol_list;
	while (strcmp(ptr->name, symbol->name) != 0)
	{
		prev = ptr;
		ptr = ptr->next;
		if (ptr == NULL) /* insert at the end of symbol list */
		{
			prev->next = symbol;
			return 1;
		}
	}
	return 0;  /* requested symbol already exists in symbol list */
}


